%%
% This script demonstrates (a) how too low number of samples influences the
% frequency resolution of a spectrum, (b) how a too low sampling frequency
% ledas to aliasing of the signal and wrong identification of salient
% frequencies.
%
% -----
% NOTE:
% -----
% The sampling frequency `Fs` determines the maximum frequency that will
% appear in the spectrum (that will be Fs/2, due to Nyquist theorem).
% If we sample only one period of the signal, i.e. only `Fs` samples, we
% get only 1 Hz resolution of the discrerte spectrum. If we need a higher
% spectral resolution, we have to sample more periods: For 0.5 Hz
% resolution we have to sample 2 periods, for 0.25 Hz we need 4 periods,
% for 0.2 Hz we need 5 periods, for 0.1 Hz we need 10 periods and so on.
%
% To demonstrate (a), set `np` to low value, e.g. 1.
% To demonstrate (b), set `Fs` to odd value 10 Hz or below (the script
% currenlty assumes that the samlping frequency is an odd integer).

clear
close all

%%
% General parameters
T = 1; % period length
np = 10;  % number of periods to consider
ppp = 1000;  % number of sample points per period for "continuous" plot 
Fs = 10;  % the sampling freqency in Hertz
t0 = 0;  % initial time sample of the signal

%%
% Generate "continuous" data for plotting
t1 = np*T;
tc = linspace(t0,t1,np*ppp+1);  % linspace includes the interval end

%%
% The freqencies of the signal
f1 = 5.1;
f2 = 5.9;

%%
% The "continuous" form of the signal
% The `T` is in fact not necessary as it is set to 1, but if we experiment
% and set `T` to another value, we will need it here as well. 
xc = sin(2*pi/T*f1*tc)+sin(2*pi/T*f2*tc);

%%
% Now let us sample the signal
% The sampling period
Ts = 1/Fs;
% The number of samples is given by the length of the observed signal, i.e.
% by the time difference from `t0` to `t1`
N = (t1-t0)*Fs;
% Determine the sampling points
k = 0:(N-1);
xs = sin(2*pi*f1*k*Ts) + sin(2*pi*f2*k*Ts);

%%
% Plot one period of the sampled signal, then all periods that have been
% sampled in order to get higher resolution, and the spectrogram
figure(1);

% Plot a single period
subplot(3,1,1);
plot(tc(1:ppp), xc(1:ppp), 'LineWidth', 1.5, 'Color', [1 0 0]);
hold on;
% Single sampled period contains exactly `Fs` samples
k_T = 0:(Fs-1);
plot(k_T*Ts, xs(1:Fs), 'o', 'Color',[0 0 1]);
hold off;
xlabel('Time [s]')
ylabel('f(t)')
title(sprintf('One period of the signal sampled with %.0f Hz', Fs));

% Plot the full length of the signal, i.e. the `np` periods
subplot(3,1,2);
plot(tc, xc, 'LineWidth', 1.5, 'Color', [1 0 0]);
hold on;
plot(k*Ts, xs, 'o', 'Color',[0 0 1]);
xlabel('Time [s]')
ylabel('f(t)')
title(sprintf('%d sampled periods for resolution %.2f Hz', np, Fs/N));

% Spectrum
subplot(3,1,3)
% The single-sided spectrogram consains (for even sampling frequencies),
% N/2 + 1 frequencies ranging from 0 to Fs/2.
freq = (0:N/2)*Fs/N;
% Two-sided spectrum
p2 = abs(fft(xs)/N);
% Single-sided spectrum: Amplitudes for koefficients with k>=1 are
% caled by 2/L (because of the symmetry)
p1 = [ p2(1), 2*p2(2:N/2+1) ];
plot(freq, p1); hold on;
stem(freq, p1); hold off;
xlim([0,freq(end)]);
ylim([min(p1), max(p1)]);
xlabel('f [Hz]');
ylabel('|P_1(f)|');
title(sprintf('%d frequency bands up to %.0f Hz, resolution %.2f Hz', N/2+1, Fs/2, Fs/N));
