%%
% This script demonstrates zero-padding (i.e. spectral interpolation) that
% helps us to artificially increase spectral resolution of a signal that
% has been sampled for a too low number of periods.
%
% -----
% NOTE:
% -----
% The sampling frequency `Fs` determines the maximum frequency that will
% appear in the spectrum (that will be Fs/2, due to Nyquist theorem).
% If we sample only one period of the signal, i.e. only `Fs` samples, we
% get only 1 Hz resolution of the discrerte spectrum. If we need a higher
% spectral resolution, we have to sample more periods, but we do not have
% them: For 0.5 Hz we would have to sample 2 periods, for 0.25 Hz we would
% need 4 periods, for 0.2 Hz it would be 5 periods, for 0.1 Hz already 10
% periods and so on.
%
% In this example we start with a signal composed of 5.1 and 5.9 Hz sine
% wave sampled at 32 Hz (i.e. the maximum recognisable frequency is 16 Hz).
% In order to get the appropriate resolution of 0.1 Hz, we need to pad the
% original signal so that it appears to have 10 periods.
%
clear
close all

%%
% General parameters
T = 1; % period length
np = 20;  % number of periods of the final padded signal
ppp = 1000;  % number of sample points per period for "continuous" plot 
Fs = 32;  % the sampling freqency in Hertz
t0 = 0;  % initial time sample of the signal

%%
% Generate "continuous" data for plotting. In this case it is only a signle
% period.
t1 = T;
tc = linspace(t0,t1,ppp+1);  % linspace includes the interval end

%%
% The freqencies of the signal
f1 = 5.1;
f2 = 5.9;

%%
% The "continuous" form of the signal
% The `T` is in fact not necessary as it is set to 1, but if we experiment
% and set `T` to another value, we will need it here as well. 
xc = sin(2*pi/T*f1*tc)+sin(2*pi/T*f2*tc);

%%
% Now let us sample the signal
% The sampling period
Ts = 1/Fs;
% The number of samples is given by the length of the observed signal, i.e.
% by the time difference from `t0` to `t1`
N = (t1-t0)*Fs;
% Determine the sampling points
k = 0:(N-1);
xs = sin(2*pi*f1*k*Ts) + sin(2*pi*f2*k*Ts);
figure(1);
axis tight

%%
% Plot one period of the sampled signal, then all periods that have been
% sampled in order to get higher resolution, and the spectrogram
figure(1);

% Plot a single period
subplot(3,1,1);
plot(tc(1:ppp), xc(1:ppp), 'LineWidth', 1.5, 'Color', [1 0 0]);
hold on;
% Single sampled period contains exactly `Fs` samples
k_T = 0:(Fs-1);
plot(k_T*Ts, xs(1:Fs), 'o', 'Color',[0 0 1]);
hold off;
xlabel('Time [s]')
ylabel('f(t)')
title(sprintf('One period of the signal sampled with %.0f Hz', Fs));

% Spectrum without zero-padding
subplot(3,1,2)
% The single-sided spectrogram contains (for even sampling frequencies),
% N/2 + 1 frequencies ranging from 0 to Fs/2.
freq = (0:N/2)*Fs/N;
% Two-sided spectrum
p2 = abs(fft(xs)/N);
% Single-sided spectrum: Amplitudes for koefficients with k>=1 are
% caled by 2/L (because of the symmetry)
p1 = [ p2(1), 2*p2(2:N/2+1) ];
plot(freq, p1); hold on;
stem(freq, p1); hold off;
xlim([0,freq(end)]);
ylim([min(p1), max(p1)]);
xlabel('f [Hz]');
ylabel('|P_1(f)|');
title(sprintf('%d frequency bands up to %.0f Hz, resolution %.2f Hz', N/2+1, Fs/2, Fs/N));


% Zero-padded spectrum with (np-1) periods of zero samples
subplot(3,1,3);
nz = (np-1)*Fs;  % number of zeros for padding the signal
L = N + nz;  % total number of samples in the zero-padded signal
% The single-sided spectrogram consains (for even sampling frequencies),
% L/2 + 1 frequencies ranging from 0 to Fs/2.
freq = (0:L/2)*Fs/L;
% Two-sided spectrum
p2z = abs(fft([xs zeros(1, nz)])/L);
% Single-sided spectrum: Amplitudes for koefficients with k>=1 are
% caled by 2/L (because of the symmetry)
p1z = [ p2z(1), 2*p2z(2:L/2+1) ];
plot(freq, p1z); hold on;
stem(freq, p1z); hold off;
xlim([0,freq(end)]);
ylim([min(p1z), max(p1z)]);
xlabel('f [Hz]')
ylabel('|P_1(f)|')
title(sprintf('Spectral interpolation, %d frequency bands, resolution %.2f Hz', L/2+1, Fs/L));

