%%
% Demonstration of audio compression using only "salient frequencies" from
% the spectrum.
%
% Use a predefined Matlab signal and try to create its "compressed" version
% that contains only a limited number of frequencies in the spectrum.
% We will see that a signekl with only cca 15% of the original spectrum has
% a quite acceptable distrotion and we my accept it as almost identical to
% the original.
%
clear
close all

% Loading the 'train.mat' file introduces variables `Fs` (sampling
% frequency in Hz) and `y` (sampled audio data).
load('train');
L = length(y);  % number of samples
freq = (0:L/2)*Fs/L;  % frequencies in the histogram

%%
% Compute fourier transform of the signal, ...
Yfft = fft(y);
% ... its two-sided spectrum, ...
p2 = abs(Yfft/L);
% ... and its single-sided spectrum: Amplitudes for coefficients with k>=1
% are scaled by 2/L (because of the symmetry).
% Note that `y` was a column vector and hence also `Yfft`, `p1` and `p2`
% are colmn vectors.
p1 = [ p2(1); 2*p2(2:L/2+1)];

%%
% Plot the spectrum
figure(1);
plot(freq, p1);
xlim([0, freq(end)]);
xlabel('f [Hz]');
ylabel('|P_1(f)|');
title('Original spectrum');

% Get the peaks in the single-sided spectrum sorted by their magnitude.
% `mypeaks` will contains the magnitude (i.e. amplitude) values, while
% `myloc` will hold the index values (we will use them to index the
% frequency data).
[mypeaks, myloc] = findpeaks(p1, 'MinPeakHeight', 0.001, 'Sort', 'desc');
% afft(myloc)
% mypeaks

% Determine f1,f2,f3 and their amplitudes
f1 = freq(myloc(1)); A1 = mypeaks(1);
f2 = freq(myloc(2)); A2 = mypeaks(2);
f3 = freq(myloc(3)); A3 = mypeaks(3);

% Possibly add three other peaks, you will need to update the code
% a bit.
%
% f4 = 704.0795; A4 = 440.5035;
% f5 = 885.9826; A5 = 978.9985;
% f6 = 1169.014; A6 = 927.9704;

t = 0:1/Fs:1.5;
ys = (A1*sin(2*pi*f1*t) + ...
      A2*sin(2*pi*f2*t) + ...
      A3*sin(2*pi*f3*t))/(A1+A2+A3);

% ys6 = (A1*sin(2*pi*f1*t) + ...
%       A2*sin(2*pi*f2*t) + ...
%       A3*sin(2*pi*f3*t) + ...
%       A4*sin(2*pi*f4*t) + ...
%       A5*sin(2*pi*f5*t) + ...
%       A6*sin(2*pi*f6*t))/(A1+A2+A3+A4+A5+A6);

%%
% Play the original file and the file containing only the three most
% prominent frequencies. Note: `sound()` is asynchronous, it sends the
% sound to the audio port and does not block until the sound has been
% played -- that is why we need to pause the execution of the script.
fprintf('Playing the orginal ...\n');
sound(y, Fs);
pause(2);
fprintf('... played.\nPlaying the synthesis of 3 most prominent frequencies ...\n');
sound(ys, Fs);
pause(2);
fprintf('... played.\n');

%%
% Signal compression by spectrum thresholding
% We will try different thresholds
thresh = [ 0.5, 0.1, 0.05, 0.01, 0.005, 0.001 ];
distor = zeros(size(thresh));  % will hold signal distortion
compr = zeros(size(thresh));  % will hold compression ratio
% Loop over all thresholds
nt = length(thresh);
for n = 1:nt
    % Select a threshold
    thr = thresh(n);
    % Remove all frequencies with amplitude below thr*max(|Yfft|)
    Ythresh = specthr(Yfft, thr);
    % Determine the compression ratio
    compr(n) = sum(abs(Ythresh)>0)/L;
    % Compute its two-sided spectrum, ...
    p2 = abs(Ythresh/L);
    % ... and its single-sided spectrum
    p1 = [ p2(1); 2*p2(2:L/2+1)];
    % Compute the inverse FFT
    yt = real(ifft(Ythresh));
    % Determine the distortion of the signal as a percentage
    distor(n) = 100*sigdstrt(y, yt);
    % Plot the thresholded spectrum
    figure(n+1);
    plot(freq, p1);
    xlim([0, freq(end)]);
    xlabel('f [Hz]');
    ylabel('|P_1(f)|');
    title(sprintf('Spectrum for threshold %f', thr));
    % Play it
    % For threshold 0.005 the result is almost identical with the original.
    fprintf('Threshold %.3f*max(amplitude), playing original ...\n', thr);
    sound(y, Fs);
    pause(2);
    fprintf('... now the compressed variant ...\n');
    sound(yt, Fs);
    pause(2);
    fprintf('... played.\n');
end

%%
% Tabular output
% Thresholds
fprintf('Thresholds:  '); fprintf('%.3f  ', thresh); fprintf('\n');
% Percentage distortion ...
fprintf('Distortion: '); fprintf('%5.2f%% ', distor);  fprintf('\n');
% Compression ratio ...
fprintf('Ratio ....:  '); fprintf('%.3f  ', compr); fprintf('\n');